import javax.management.ObjectName;
import javax.management.Query;
import javax.management.openmbean.CompositeData;

import fr.ippon.groovy.util.WebLoMBean;

def mbs = WebLoMBean.getDomainMBeanServer('t3://localhost:7001', 'weblogic', 'weblogic')

def destFilter = javax.management.Query.with {
	match(attr('Name'),value('*@MyFirstDistributedQueue'))
}

def destinationMBeanNames = mbs.queryNames(new ObjectName('com.bea:Type=JMSDestinationRuntime,*'), destFilter)
def destinationMbeans = destinationMBeanNames.collect { new GroovyMBean(mbs, it) }

def selector = "JMSType = 'MyCustomJMSType'"
	
def BIT_TO_DESCRIPTION = [
    (1 << 0)  : "Visible",
	(1 << 1)  : "Pending sent",
	(1 << 2)  : "Pending receive",
	(1 << 3)  : "In transaction",
	(1 << 4)  : "Bit 2^4 (?)",
    (1 << 5)  : "Delayed",
	(1 << 6)  : "Bit 2^6 (?)",
	(1 << 7)  : "Bit 2^7 (?)"
]

def extractMessageDetail = { state, messageXMLText ->
	// dcodage du statut : 
	def stateElement = [];
	BIT_TO_DESCRIPTION.each { bit, description ->
		if(state & bit) {
			stateElement << description
		}
	}
	def stateDescription = stateElement.join(", ")
	println "State : $stateDescription"
	
	// parsing du xml : 
	def parser = new XmlParser(false,false)  // pas de gestion des namespace ici
	def xml = parser.parseText(messageXMLText)

	def header = xml.'mes:Header'[0]
	
	// headers :
	println "Headers : "
	header.'*'.findAll { it.name() != 'mes:Properties' }.each { node ->
		def headerName = node.name()[4..-1] // suppression du prfixe mes: ...
		def value = node.text()
		printf "  %-20s : %s%n" , headerName, value
	}
	
	// custom properties :
	println "Properties : "
	header.'mes:Properties'[0].'mes:property'.each { node ->
		def propertyName = node.'@name'
		def valueNode = node.'*'[0]
		// le type peut-tre en particulier : String, Boolean, Int, Long, ...
		def type = valueNode.name()[4..-1] // suppression du prfixe mes: ...
		
		def value = valueNode.text()
		
		printf "  %-20s (type : %5s) : %s%n" , propertyName, type, value
	}
	
	// body :
	def payload = xml.'mes:Body'[0]
	
	// Le noeud contenant la payload peut-tre un mes:Text ou un mes:Object, ou ... 
	
	
	if(payload.'mes:Text') {
		println "Content : (this is a text message) : "
		println payload.'mes:Text'[0].text();
	} else {
		def payloadType = payload.'*'[0].name()[4..-1]
		println "Content : The type of this message is '$payloadType', extraction not implemented"
	}
	
}

destinationMbeans.each { mbean ->

	def timeout = 1000
	def cursor = mbean.getMessages(selector, timeout)

	def nbMsg = mbean.getCursorSize(cursor)
	println "Found $nbMsg on queue $mbean.Name"
	
	def batchSize = 10
	def readMessage = 0
	while(readMessage < nbMsg) {
		def items = mbean.getNext(cursor,batchSize)
		items.each { CompositeData item ->
			++readMessage
			println "\nMsg n$readMessage :"
			
			def msgHandle = item['Handle']
			def state = item['State']
			
//			def itemWithBody = mbean.getMessage(cursor,msgHandle) // ok sur WebLo 10 mais bug groovy sur WebLo 9 (http://jira.codehaus.org/browse/GROOVY-2807)
			def itemWithBody = mbean.server().invoke(
			        mbean.name(),
					'getMessage',
					[cursor,msgHandle] as Object[],
					['java.lang.String', 'java.lang.Long'] as String[])

			def messageXMLText = itemWithBody['MessageXMLText']
			extractMessageDetail(state, messageXMLText)
		}
	}
	mbean.closeCursor(cursor)
}
